# QTRES - Process a single TRES image from a raw data file to a spectrum
# February 13, 2008
# By Doug Mink, SAO Telescope Data Center

procedure qtres (image)

string	image=""	{prompt="Image or sequence number to process"}
string	datedir=""	{prompt="Date for image"}
string	instdir=""	{prompt="Directory containing date directory"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixpreamp=yes	{prompt="Fix preamp time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
string  normfile=""     {prompt="Pixel to pixel tmakeflat normalization file"}
string  normdir="tresdata$" {prompt="Directory for normalization file"}
string	aproot="flat"	{prompt="Root file name for aperture reference file"}
file	apdir="tresdata$" {prompt="Directory for aperture reference file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$" {prompt="Directory for calibration lamp file"}
int	plotorder=1	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=yes	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	ims, imin, impath, cdir
file	imout
string	imroot, ddir, croot
int	pl, n
bool	vb, dt, sv, sr, disp

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	pl = plotorder
	ddir = datedir
	imin = image
	cdir = compdir
	croot = comproot
	disp = disperse

	if (imin == "") {
	    print ("QTRES: Reduce a single TRES image in a different directory")
	    print ("Usage: qtres image with both amps to be corrected")
	    return
	    }

# Copy image to current directory
	if (instdir != "") {
	    fixdir (instdir)
	    impath = fixdir.newdir

# If a number is given, prepend appropriate zeroes and wildcard
	    isnum (imin)
	    if (isnum.fpcode > 0) {
		n = int (imin)
		if (n < 10)
		    imin = "000" // n // ".*"
		else if (n < 100)
		    imin = "00" // n // ".*"
		else if (n < 1000)
		    imin = "0" // n // ".*"
		else
		    imin = "0" // n // ".*"
		}

# If file extension is missing, add wildcard
	    else {
		smatch (imin, ".fits")
		if (smatch.found == no)
		    imin = imin // "*"
		}
	    if (ddir != "")
		impath = impath // ddir // "/" // imin
	    else
		impath = impath // imin
	    filepath (impath)
	    if (!access (filepath.files)) {
		print ("QTRES: Cannot read file ",filepath.files)
		}
	    filename (filepath.files)
	    if (ddir != "")
		imout = ddir // "." // filename.filename
	    else
		imout = filename.filename
	    if (vb) {
		print ("QTRES: Copying file ",filepath.files," to ", imout)
		}
	    if (!access (imout))
		copy (impath, imout)
	    }
	else {
	    imout = imin
	    }

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = flatten
	tproc.extract = yes
	tproc.disperse = disperse
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.cosmic = no
	tproc.aproot = aproot
	tproc.apdir = apdir
	tproc.refroot = croot
	tproc.refdir = cdir
	tproc.recenter = recenter
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.saveraw = sr
	tproc.verbose = vb
	tproc (imout)

# Plot spectra if desired
	if (pl > 0) {
	    fileroot (imout)
	    imroot = fileroot.rootname
	    ims = imroot // "_fib1.ec.fits"
	    if (access (ims)) {
		splot (ims,pl)
		}
	    ims = imroot // "_fib2.ec.fits"
	    if (access (ims)) {
		splot (ims,pl)
		}
	    ims = imroot // "_fib3.ec.fits"
	    if (access (ims)) {
		splot (ims,pl)
		}
	    }
end

# Oct 12 2007	New task for quick look
# Dec 11 2007	Remove old declaration of instdir from before it was a parameter

# Jan 16 2008	Add fixpreamp parameter
# Jan 17 2008	Use tproc instead of tpreproc and tflatten
# Jan 23 2008	Add plotorder parameter to plot spectrum at end
# Jan 23 2008	Add recenter to make recentering during extraction optional
# Jan 23 2008	Add comproot parameter and wavelength calibration
# Jan 28 2008	Add disperse parameter to make wavelength optional
# Jan 30 2008	Use tdisp to add wavelength solution
# Jan 30 2008	Add parameters apdir and aproot for extraction templates
# Feb 12 2008	Use TPROC for everything; drop calls to textract and tdisperse
# Feb 13 2008	Change parameter imdate to datedir
