# File tres/Scripts/tcal.cl
# January 29, 2008
# By Doug Mink

#  TCAL -- Make a local id database file for a calibration spectrum

procedure tcal (spectrum)

string  spectrum=""       {prompt="Images to check"}

string	refroot="comp"	{prompt="Root name for comparison reference file"}
string	refdir="tresdata$" {prompt="Directory for comparison reference file"}
bool	saveold = yes	{prompt="Save original file to Nonorm/ (y or n)"}
bool    verbose=yes	{prompt="Print process progress (y or n)"}
bool    debug=no	{prompt="Print extra information (y or n)"}

begin

    string imin, calref, history
    file imref, impix, newfile, rdir
    bool vb, db

    imin = spectrum
    rdir = refdir
    vb = verbose
    db = debug

    tidref (imin, refroot=refroot, refdir=rdir, verbose=vb, debug=db)
    imref = tidref.reffile

    if (!deftask ("pxcsao")) {
	rvsao (motd=no)
	}

    pxcsao.spectra = imin
    pxcsao.specnum = "1-51"
    pxcsao.specband = 0
    pxcsao.specdir = ""
    pxcsao.correlate = "pixel"
    pxcsao.templates = imref
    pxcsao.tempnum = "1-51"
    pxcsao.tempband = 1
    pxcsao.tempdir = rdir
    pxcsao.echelle = yes
    pxcsao.st_lambda = INDEF
    pxcsao.end_lambda = INDEF
    pxcsao.obj_plot = no
    pxcsao.xcor_plot = no
    pxcsao.xcor_file = no
    pxcsao.fixbad = no
    pxcsao.badlines = "badlines.dat"
    pxcsao.s_emchop = "no"
    pxcsao.t_emchop = "no"
    pxcsao.s_abs_reject = 100.
    pxcsao.s_em_reject = 2.
    pxcsao.t_abs_reject = 0.
    pxcsao.t_em_reject = 0.
    pxcsao.bell_window = 0.05
    pxcsao.renormalize = no
    pxcsao.ncols = 4096
    pxcsao.interp_mode = "spline3"
    pxcsao.zeropad = "no"
    pxcsao.low_bin = 5
    pxcsao.top_low = 20
    pxcsao.top_nrun = 125
    pxcsao.nrun = 250
    pxcsao.vel_init = "zero"
    pxcsao.czguess = 0.
    pxcsao.nzpass = 0
    pxcsao.tshift = 0.
    pxcsao.svel_corr = "none"
    pxcsao.tvel_corr = "none"
    pxcsao.pkmode = 1
    pxcsao.pkfrac = 0.5
    pxcsao.pksrch = 25
    pxcsao.minvel = -100.
    pxcsao.maxvel = 100.
    pxcsao.report_mode = 2
    pxcsao.logfiles = "STDOUT,xcsao.log"
    pxcsao.save_vel = no
    pxcsao.rvcheck = no
    pxcsao.archive = no
    pxcsao.nsmooth = 0
    pxcsao.cvel = INDEF
    pxcsao.dvel = INDEF
    pxcsao.ablines = "ablines.dat"
    pxcsao.emlines = "emlines.dat"
    pxcsao.linedir = "rvsao$lib/"
    pxcsao.dispmode = 1
    pxcsao.displot = yes
    pxcsao.device = "stdgraph"
    pxcsao.vel_plot = "correlation"
    pxcsao.curmode = no
    pxcsao.hardcopy = no
    pxcsao.plotter = "stdplot"
    pxcsao.besttemp = ""
    pxcsao.velocity = 0.0
    pxcsao.velerr = 0.0
    pxcsao.r = 0.0
    pxcsao.temp_plot = no
    pxcsao.contsub_plot = no
    pxcsao.apodize_plot = no
    pxcsao.fft_plot = no
    pxcsao.tfft_plot = no
    pxcsao.uxcor_plot = no
    pxcsao.nsum = 1
    pxcsao.debug = db


# Cross correlate with reference calibration spectrum
    pxcsao (imin)

# Copy reference database/id file with shift from correlation
    tmodid.indir = rdir
    tmodid.outdir = ""
    tmodid.shift = pxcsao.velocity
    hedit (imin, "TIDSHIFT", pxcsao.velocity, add+, update+,verify-)
    tmodid.scale = 1.0
    tmodid.keepfit = no
    tmodid.verbose = vb
    tmodid.debug = db
    tmodid (imref, imin)

# Adjust line fit
    fileroot (imin)
    calref = fileroot.rootname
    hedit (imin, "REFSPEC1", calref, add+, update+,verify-)
    ecidentify.coordlist = "linelists$thar.dat"
    ecidentify (imin)


    fileroot (calref)
    impix = fileroot.rootname // "p.ec.fits"
    rename (imin, impix)
    dispcor (impix, imin, linearize=no)

# Add history line including time
    gettime ()
    history = gettime.date // " " // gettime.time // " dispersion fit from " // calref
    hedit (imin, "TCAL", history, add+, verify-)

# Move original file to Nodisp directory
    if (saveold) {
	if (!access ("Nodisp/")) {
	    mkdir ("Nodisp")
	    }
	newfile = "Nodisp/" // imin
	if (access (newfile)) {
	    imdelete (newfile)
	    }
	rename (impix, newfile)
	if (vb) {
	    print ("TCAL: Original ", imin, " => Nodisp/")
	    }
	}

end

# Jan 28 2008	New task
# Jan 29 2008	Add saveold to save undispersed spectrum to Nodisp/
# Jan 29 2008	Write TCAL FITS parameter to output file
